"""Classes for data enhancements Trollflow based Trollduction"""

import numpy as np
import logging

from trollflow.workflow_component import AbstractWorkflowComponent


class Pansharpener(AbstractWorkflowComponent):

    """Pansharpen the configured channels"""

    logger = logging.getLogger("Pansharpener")

    def __init__(self):
        super(Pansharpener, self).__init__()

    def pre_invoke(self):
        """Pre-invoke"""
        pass

    def invoke(self, context):
        """Invoke"""
        glbl = context["content"]
        # Read list of channels to be sharpened
        pan_chans = context["pan_sharpen_chans"]
        self.logger.info("Applying pansharpening to channels: %s",
                         str(pan_chans))
        # Check if the original data should be overwritten (default)
        # or create a new channel named "pan_"+chan.name
        try:
            overwrite = context["overwrite"]
        except KeyError:
            overwrite = True

        if overwrite:
            self.logger.info("Original data will be overwritten.")
        else:
            self.logger.info("Pansharpened channels will be named with "
                             "'pan_' prefix.")

        # Apply pansharpening
        pansharpen(glbl, pan_chans, overwrite)

        # Put enhanced data to output queue
        context["output_queue"].put(glbl)

    def post_invoke(self):
        """Post-invoke"""
        pass


def pansharpen(glbl, pan_chans, overwrite=True):
    """Apply pansharpening"""
    hrv = glbl["HRV"].data

    # Pad HRV data, if shape isn't divisible by three
    hrv_orig_shape = hrv.shape
    y_rem = hrv_orig_shape[0] % 3
    x_rem = hrv_orig_shape[1] % 3
    if y_rem != 0:
        hrv = np.vstack((hrv, hrv[-(3 - y_rem):, :]))
    if x_rem != 0:
        hrv = np.hstack((hrv, hrv[:, -(3 - x_rem):]))

    shape = hrv.shape
    highresr = np.roll(np.roll(hrv, -1, axis=0), -2, axis=1)
    lowres = highresr.reshape(
        [shape[0] / 3, 3, shape[1] / 3, 3]).mean(3).mean(1)
    highres = np.ma.repeat(np.ma.repeat(lowres, 3, axis=0), 3, axis=1)
    ratio = highresr / highres
    # Cut to original HRV shape
    # ratio = ratio[0:hrv_orig_shape[0], 0:hrv_orig_shape[1]]

    # Cut to original HRV shape
    glbl["HRV"].data = highresr[0:hrv_orig_shape[0], 0:hrv_orig_shape[1]]

    for chan_name in pan_chans:
        chan = 1.0 * glbl[chan_name]

        # Pad data if shape isn't divisible by three
        shape = chan.data.shape
        y_rem = shape[0] % 3
        x_rem = shape[1] % 3
        if y_rem != 0:
            chan.data = np.vstack((chan.data, chan.data[-(3 - y_rem):, :]))
        if x_rem != 0:
            chan.data = np.hstack((chan.data, chan.data[:, -(3 - x_rem):]))

        # Replicate data
        chan.data = np.ma.repeat(np.ma.repeat(chan.data, 3, axis=0), 3, axis=1)

        # Adjust to original HRV shape
        shape = chan.data.shape
        y_diff, x_diff = ratio.shape[0], ratio.shape[1]
        if y_diff != 0:
            # Add pixels
            if y_diff > 0:
                chan.data = np.vstack((chan.data, chan.data[-y_diff:, :]))
            # Cut extra pixels
            else:
                chan.data = chan.data[:-y_diff, :]
        if x_diff != 0:
            # Add pixels
            if x_diff > 0:
                chan.data = np.hstack((chan.data, chan.data[:, -x_diff:]))
            # Cut extra pixels
            else:
                chan.data = chan.data[:, :-x_diff]

        # Cut the data to original HRV shape and apply sharpening
        chan.data = chan.data[0:hrv_orig_shape[0], 0:hrv_orig_shape[1]]
        chan.data *= ratio[0:hrv_orig_shape[0], 0:hrv_orig_shape[1]]

        # Either overwrite the data, or create a new channel
        if overwrite:
            glbl[chan_name].data = chan.data
        else:
            chan_name = "pan_" + chan_name
            chan.name = chan_name
            glbl.channels.append(chan)

        # Replace area definition with HRV area def
        glbl[chan_name].area = glbl["HRV"].area



"""Class for reading satellite data for Trollflow based Trollduction
using satpy"""

import logging
import yaml
import time

from trollflow_sat import utils
from trollflow.workflow_component import AbstractWorkflowComponent
from satpy import Scene


class SceneLoader(AbstractWorkflowComponent):

    """Creates a scene object from a message and loads the required channels.
    """

    logger = logging.getLogger("SceneLoader")

    def __init__(self):
        super(SceneLoader, self).__init__()

    def pre_invoke(self):
        """Pre-invoke"""
        pass

    def invoke(self, context):
        """Invoke"""
        # Set locking status, default to False
        self.use_lock = context.get("use_lock", False)
        self.logger.debug("Locking is used in resampler: %s",
                          str(self.use_lock))
        if self.use_lock:
            self.logger.debug("Compositor acquires lock of previous "
                              "worker: %s", str(context["prev_lock"]))
            utils.acquire_lock(context["prev_lock"])

        instruments = context.get("instruments", None)
        if instruments is None:
            utils.release_locks([context["lock"], context["prev_lock"]],
                                log=self.logger.error,
                                log_msg="No instruments configured!")
            return

        with open(context["product_list"], "r") as fid:
            product_config = yaml.load(fid)
        msg = context['content']

        global_data = self.create_scene_from_message(msg, instruments)
        if global_data is None:
            utils.release_locks([context["lock"], context["prev_lock"]],
                                log=self.logger.info,
                                log_msg="Unable to create Scene, " +
                                "skipping data")
            return

        monitor_topic = context.get("monitor_topic", None)
        if monitor_topic is not None:
            nameservers = context.get("nameservers", None)
            port = context.get("port", 0)
            service = context.get("service", None)
            monitor_metadata = utils.get_monitor_metadata(msg.data,
                                                          status="start",
                                                          service=service)
            utils.send_message(monitor_topic,
                               "monitor",
                               monitor_metadata,
                               nameservers=nameservers, port=port)

        # use_extern_calib = product_config["common"].get("use_extern_calib",
        #                                                 "False")

        for group in product_config["groups"]:
            # Set lock if locking is used
            if self.use_lock:
                self.logger.debug("Compositor acquires own lock %s",
                                  str(context["lock"]))
                utils.acquire_lock(context["lock"])

            if "collection_area_id" in msg.data:
                if group != msg.data["collection_area_id"]:
                    utils.release_locks([context["lock"]],
                                        log=self.logger.debug,
                                        log_msg="Collection not for this " +
                                        "area, skipping")
                    continue

            composites = utils.get_satpy_group_composite_names(product_config,
                                                               group)
            prev_reqs = {itm.name for itm in global_data.datasets}
            reqs_to_unload = prev_reqs - composites
            if len(reqs_to_unload) > 0:
                self.logger.debug("Unloading unnecessary channels: %s",
                                  str(sorted(reqs_to_unload)))
                global_data.unload(list(reqs_to_unload))
            self.logger.info("Loading required data for this group: %s",
                             ', '.join(sorted(composites)))

            # use_extern_calib=use_extern_calib
            global_data.load(composites)
            context["output_queue"].put(global_data)

            if utils.release_locks([context["lock"]]):
                self.logger.debug("Compositor releases own lock %s",
                                  str(context["lock"]))
                # Wait 1 second to ensure next worker has time to acquire the
                # lock
                time.sleep(1)

        del global_data
        global_data = None

        # Wait until the lock has been released downstream
        if self.use_lock:
            utils.acquire_lock(context["lock"])
            utils.release_locks([context["lock"]])

        if monitor_topic is not None:
            monitor_metadata["status"] = "completed"
            utils.send_message(monitor_topic,
                               "monitor",
                               monitor_metadata,
                               nameservers=nameservers,
                               port=port)

        # After all the items have been processed, release the lock for
        # the previous step
        utils.release_lock([context["prev_lock"]], log=self.logger.debug,
                           log_msg="Compositor releses lock of previous "
                           "worker")

    def post_invoke(self):
        """Post-invoke"""
        pass

    def create_scene_from_message(self, msg, instruments):
        """Parse the message *msg* and return a corresponding MPOP scene.
        """
        if msg.type in ["file", 'collection', 'dataset']:
            return self.create_scene_from_mda(msg.data, msg.type, instruments)

    def create_scene_from_mda(self, mda, msg_type, instruments):
        """Read the metadata *mda* and return a corresponding MPOP scene.
        """
        start_time = (mda.get('start_time') or
                      mda.get('nominal_time') or
                      None)
        end_time = mda.get('end_time') or None

        platform_name = mda["platform_name"]

        if isinstance(mda['sensor'], (list, tuple, set)):
            sensor = mda['sensor'][0]
        else:
            sensor = mda['sensor']

        if sensor not in instruments:
            self.logger.debug("Unknown sensor, skipping data.")
            return None

        if msg_type == "dataset":
            filenames = []
            for dset in mda["dataset"]:
                filenames.append(dset["uri"])
        elif msg_type == "collection":
            raise NotImplementedError
        else:
            filenames = mda['uri']

        if not isinstance(filenames, (list, set, tuple)):
            filenames = [filenames]

        # Create satellite scene
        global_data = Scene(platform_name=platform_name,
                            sensor=sensor,
                            start_time=start_time,
                            end_time=end_time,
                            filenames=filenames)

        global_data.info.update(mda)

        self.logger.debug("SCENE: %s", str(global_data.info))

        return global_data



"""Classes for generating image composites for Trollflow based
Trollduction using satpy"""

import logging
import yaml
import time

from trollflow.workflow_component import AbstractWorkflowComponent
from trollflow_sat import utils


class Resampler(AbstractWorkflowComponent):

    """Creates resampled local area scenes."""

    logger = logging.getLogger("Resampler")

    def __init__(self):
        super(Resampler, self).__init__()

    def pre_invoke(self):
        """Pre-invoke"""
        pass

    def invoke(self, context):
        """Invoke"""
        # Set locking status, default to False
        self.use_lock = context.get("use_lock", False)
        self.logger.debug("Locking is used in resampler: %s",
                          str(self.use_lock))
        if self.use_lock:
            self.logger.debug("Compositor acquires lock of previous "
                              "worker: %s", str(context["prev_lock"]))
            utils.acquire_lock(context["prev_lock"])

        glbl = context["content"]
        with open(context["product_list"], "r") as fid:
            product_config = yaml.load(fid)

        # Handle config options
        kwargs = {}
        try:
            kwargs['precompute'] = context["precompute"]
        except KeyError:
            kwargs['precompute'] = False
        self.logger.debug("Setting precompute to %s",
                          str(kwargs['precompute']))
        try:
            kwargs['nprocs'] = context["nprocs"]
        except KeyError:
            kwargs['nprocs'] = 1
        self.logger.debug("Using %d CPUs for resampling.", kwargs['nprocs'])
        try:
            kwargs['resampler'] = context["proj_method"]
        except KeyError:
            kwargs['resampler'] = "nearest"
        self.logger.debug(
            "Using resampling method: '%s'.", kwargs['resampler'])
        try:
            area = glbl.area.area_id
            area_config = product_config["product_list"][area]
            kwargs['radius_of_influence'] = \
                area_config.get("srch_radius", context["radius"],
                                10000.)
        except (AttributeError, KeyError):
            kwargs['radius_of_influence'] = 10000.

        if kwargs['radius_of_influence'] is None:
            self.logger.debug("Using default search radius.")
        else:
            self.logger.debug("Using search radius %d meters.",
                              int(kwargs['radius_of_influence']))

        prod_list = product_config["product_list"]
        for area_id in prod_list:
            # Set lock if locking is used
            if self.use_lock:
                self.logger.debug("Resampler acquires own lock %s",
                                  str(context["lock"]))
                utils.acquire_lock(context["lock"])
            if area_id not in glbl.info["areas"]:
                utils.release_locks([context["lock"]])
                continue

            # Reproject only needed channels
            dataset_names = \
                utils.get_satpy_area_composite_names(product_config, area_id)
            dataset_ids = [ds_id for ds_id in glbl.datasets.keys()
                           if ds_id.name in dataset_names]
            if area_id == "satproj":
                self.logger.info("Using satellite projection")
                lcl = glbl
            else:
                self.logger.info("Resampling time slot %s to area %s",
                                 glbl.info["start_time"], area_id)
                lcl = glbl.resample(area_id, datasets=dataset_ids,
                                    **kwargs)
            lcl.info["product_config"] = product_config
            lcl.info["area_id"] = area_id
            lcl.info["products"] = prod_list[area_id]['products']
            lcl.info["dataset_ids"] = dataset_ids
            self.logger.debug("Inserting lcl (area: %s, start_time: %s) "
                              "to writer's queue",
                              area_id, str(lcl.info["start_time"]))
            context["output_queue"].put(lcl)
            del lcl
            lcl = None

            if utils.release_locks([context["lock"]]):
                self.logger.debug("Resampler releases own lock %s",
                                  str(context["lock"]))
                # Wait 1 second to ensure next worker has time to acquire the
                # lock
                time.sleep(1)

        # Wait until the lock has been released downstream
        if self.use_lock:
            utils.acquire_lock(context["lock"])
            utils.release_locks([context["lock"]])

        # After all the items have been processed, release the lock for
        # the previous step
        utils.release_locks([context["prev_lock"]], log=self.logger.debug,
                            log_msg="Resampler releses lock of previous " +
                            "worker: %s" % str(context["prev_lock"]))

    def post_invoke(self):
        """Post-invoke"""
        pass



"""Classes for generating image composites for Trollflow based Trollduction"""

import logging
import yaml
import time

from trollflow.workflow_component import AbstractWorkflowComponent
from trollflow_sat import utils


class Resampler(AbstractWorkflowComponent):

    """Creates resampled local area scenes."""

    logger = logging.getLogger("Resampler")

    def __init__(self):
        super(Resampler, self).__init__()

    def pre_invoke(self):
        """Pre-invoke"""
        pass

    def invoke(self, context):
        """Invoke"""
        # Set locking status, default to False
        self.use_lock = context.get("use_lock", False)
        self.logger.debug("Locking is used in compositor: %s",
                          str(self.use_lock))
        if self.use_lock:
            self.logger.debug("Resampler acquires lock of previous worker: %s",
                              str(context["prev_lock"]))
            utils.acquire_lock(context["prev_lock"])

        glbl = context["content"]
        with open(context["product_list"], "r") as fid:
            product_config = yaml.load(fid)

        # Handle config options
        try:
            precompute = context["precompute"]
            self.logger.debug("Setting precompute to %s", str(precompute))
        except KeyError:
            precompute = False
        try:
            nprocs = context["nprocs"]
            self.logger.debug("Using %d CPUs for resampling.", nprocs)
        except KeyError:
            nprocs = 1
        try:
            proj_method = context["proj_method"]
            self.logger.debug("Using resampling method: '%s'.", proj_method)
        except KeyError:
            proj_method = "nearest"
        try:
            radius = context["radius"]
        except (AttributeError, KeyError):
            radius = None

        if radius is None:
            self.logger.debug("Using default search radius.")
        else:
            self.logger.debug("Using search radius %d meters.", int(radius))

        prod_list = product_config["product_list"]
        for area_id in prod_list:
            # Set lock if locking is used
            if self.use_lock:
                self.logger.debug("Resampler acquires own lock %s",
                                  str(context["lock"]))
                utils.acquire_lock(context["lock"])
            if area_id not in glbl.info["areas"]:
                utils.release_locks([context["lock"]])
                continue

            # Reproject only needed channels
            channels = utils.get_prerequisites_yaml(glbl,
                                                    prod_list,
                                                    [area_id, ])
            if area_id == "satproj":
                self.logger.info("Using satellite projection")
                lcl = glbl
            else:
                self.logger.info("Resampling to area %s", area_id)
                lcl = glbl.project(area_id, channels=channels,
                                   precompute=precompute,
                                   mode=proj_method, radius=radius,
                                   nprocs=nprocs)
            lcl.info["area_id"] = area_id
            lcl.info["products"] = prod_list[area_id]['products']
            context["output_queue"].put(lcl)
            del lcl
            lcl = None
            if utils.release_locks([context["lock"]]):
                self.logger.debug("Resampler releases own lock %s",
                                  str(context["lock"]))
                # Wait 1 second to ensure next worker has time to acquire the
                # lock
                time.sleep(1)

        # Wait until the lock has been released downstream
        if self.use_lock:
            utils.acquire_lock(context["lock"])
            utils.release_locks([context["lock"]])

        # After all the items have been processed, release the lock for
        # the previous step
        utils.release_locks([context["prev_lock"]], log=self.logger.debug,
                            log_msg="Resampler releses lock of previous " +
                            "worker: %s" % str(context["prev_lock"]))

    def post_invoke(self):
        """Post-invoke"""
        pass



"""Classes for generating image composites for Trollflow based Trollduction"""

import logging
import yaml
import time
try:
    from ConfigParser import NoSectionError
except ImportError:
    from configparser import NoSectionError

from trollflow.workflow_component import AbstractWorkflowComponent
from trollflow_sat import utils


class CompositeGenerator(AbstractWorkflowComponent):

    """Creates composites from a product config."""

    logger = logging.getLogger("Compositor")

    def __init__(self):
        super(CompositeGenerator, self).__init__()

    def pre_invoke(self):
        """Pre-invoke"""
        pass

    def invoke(self, context):
        """Invoke"""
        # Set locking status, default to False
        self.use_lock = context.get("use_lock", False)
        self.logger.debug("Locking is used in compositor: %s",
                          str(self.use_lock))
        if self.use_lock:
            self.logger.debug("Compositor acquires lock of previous "
                              "worker: %s", str(context["prev_lock"]))
            utils.acquire_lock(context["prev_lock"])

        data = context["content"]

        with open(context["product_list"], "r") as fid:
            product_config = yaml.load(fid)

        for prod in data.info["products"]:
            # Set lock if locking is used
            if self.use_lock:
                self.logger.debug("Compositor acquires own lock %s",
                                  str(context["lock"]))
                utils.acquire_lock(context["lock"])

            if utils.bad_sunzen_range(data.area, product_config,
                                      data.info["area_id"], prod,
                                      data.time_slot):
                utils.release_locks([context["lock"]], log=self.logger.info,
                                    log_msg="Sun zenith angle out of valid " +
                                    "range, skipping")
                continue

            self.logger.info("Creating composite %s", prod)
            try:
                func = getattr(data.image, prod)
                img = func()
                if img is None:
                    utils.release_locks([context["lock"]])
                    continue
                img.info.update(data.info)
            except (AttributeError, KeyError, NoSectionError):
                utils.release_locks([context["lock"]],
                                    log=self.logger.warning,
                                    log_msg="Invalid composite, skipping")
                continue

            # Get filename and product name from product config
            fnames, productname = utils.create_fnames(
                data.info, product_config, prod)

            if fnames is None:
                self.logger.error("Could not generate valid filename(s), "
                                  "product not saved!")
            else:
                img.info["fnames"] = fnames
                img.info["productname"] = productname
                context["output_queue"].put(img)
            del img
            img = None

            if self.use_lock:
                self.release_locks([context["lock"]], log=self.logger.debug,
                                   log_msg="Compositor releases own lock %s" %
                                   str(context["lock"]))
                # Wait 1 second to ensure next worker has time to acquire the
                # lock
                time.sleep(1)

        # Wait until the lock has been released downstream
        if self.use_lock:
            utils.acquire_lock(context["lock"])
            utils.release_locks([context["lock"]])

        # After all the items have been processed, release the lock for
        # the previous step
        utils.release_locks([context["prev_lock"]], log=self.logger.debug,
                            log_msg="Compositor releses lock of "
                            "previous worker: %s" %
                            str(context["prev_lock"]))

    def post_invoke(self):
        """Post-invoke"""
        pass




"""Class for reading satellite data for Trollflow based Trollduction"""

import logging
import yaml
import time
from urlparse import urlparse
from struct import error as StructError

from trollflow_sat import utils
from trollflow.workflow_component import AbstractWorkflowComponent
from mpop.satellites import GenericFactory as GF


class SceneLoader(AbstractWorkflowComponent):

    """Creates a scene object from a message and loads the required channels.
    """

    logger = logging.getLogger("SceneLoader")

    def __init__(self):
        super(SceneLoader, self).__init__()

    def pre_invoke(self):
        """Pre-invoke"""
        pass

    def invoke(self, context):
        """Invoke"""
        # Set locking status, default to False
        self.use_lock = context.get("use_lock", False)
        self.logger.debug("Locking is used in compositor: %s",
                          str(self.use_lock))
        if self.use_lock:
            self.logger.debug("Scene loader acquires lock of previous "
                              "worker: %s", str(context["prev_lock"]))
            utils.acquire_lock(context["prev_lock"])

        instruments = context.get("instruments", None)
        if instruments is None:
            utils.release_locks([context["lock"], context["prev_lock"]],
                                log=self.logger.error,
                                log_msg="No instruments configured!")
            return

        with open(context["product_list"], "r") as fid:
            product_config = yaml.load(fid)

        # Read message
        msg = context['content']

        global_data = self.create_scene_from_message(msg, instruments)

        if global_data is None:
            utils.release_locks([context["lock"], context["prev_lock"]],
                                log=self.logger.info,
                                log_msg="Unable to create Scene, " +
                                "skipping data")
            return

        monitor_topic = context.get("monitor_topic", None)
        if monitor_topic is not None:
            nameservers = context.get("nameservers", None)
            port = context.get("port", 0)
            service = context.get("service", None)
            monitor_metadata = utils.get_monitor_metadata(msg.data,
                                                          status="start",
                                                          service=service)
            utils.send_message(monitor_topic,
                               "monitor",
                               monitor_metadata,
                               nameservers=nameservers, port=port)

        fnames = get_data_fnames(msg)
        use_extern_calib = product_config["common"].get("use_extern_calib",
                                                        "False")
        keywords = {'use_extern_calib': use_extern_calib,
                    'load_again': True}
        if fnames is not None:
            keywords['filename'] = fnames
            self.logger.debug("Loading from files: %s", str(fnames))

        for group in product_config["groups"]:
            # Set lock if locking is used
            if self.use_lock:
                self.logger.debug("Scene loader acquires own lock %s",
                                  str(context["lock"]))
                utils.acquire_lock(context["lock"])

            if "collection_area_id" in msg.data:
                if group != msg.data["collection_area_id"]:
                    utils.release_locks([context["lock"]],
                                        log=self.logger.debug,
                                        log_msg="Collection not for this " +
                                        "area, skipping")
                    continue

            grp_area_def_names = product_config["groups"][group]

            self.logger.debug("Loading data for group %s with areas %s",
                              group, str(grp_area_def_names))

            reqs = utils.get_prerequisites_yaml(global_data,
                                                product_config["product_list"],
                                                grp_area_def_names)

            self.logger.info("Loading required channels for this group: %s",
                             str(sorted(reqs)))

            try:
                if "satproj" in grp_area_def_names:
                    try:
                        del keywords["area_def_names"]
                    except KeyError:
                        pass
                    global_data.load(reqs, **keywords)
                else:
                    keywords["area_def_names"] = grp_area_def_names
                    global_data.load(reqs, **keywords)
            except (StructError, IOError):
                utils.release_locks([context["lock"]], log=self.logger.error,
                                    log_msg="Data could not be read!")
                continue

            global_data.info["areas"] = grp_area_def_names
            context["output_queue"].put(global_data)

            if utils.release_locks([context["lock"]]):
                self.logger.debug("Scene loader releases own lock %s",
                                  str(context["lock"]))
                # Wait 1 second to ensure next worker has time to acquire the
                # lock
                time.sleep(1)

        del global_data
        global_data = None

        # Wait until the lock has been released downstream
        if self.use_lock:
            utils.acquire_lock(context["lock"])
            utils.release_locks([context["lock"]])

        if monitor_topic is not None:
            monitor_metadata["status"] = "completed"
            utils.send_message(monitor_topic,
                               "monitor",
                               monitor_metadata,
                               nameservers=nameservers,
                               port=port)

        # After all the items have been processed, release the lock for
        # the previous step
        utils.release_lock([context["prev_lock"]], log=self.logger.debug,
                           log_msg="Scene loader releses lock of " +
                           "previous worker")

    def post_invoke(self):
        """Post-invoke"""
        pass

    def create_scene_from_message(self, msg, instruments):
        """Parse the message *msg* and return a corresponding MPOP scene.
        """
        if msg.type in ["file", 'collection', 'dataset']:
            return self.create_scene_from_mda(msg.data, instruments)

    def create_scene_from_mda(self, mda, instruments):
        """Read the metadata *mda* and return a corresponding MPOP scene.
        """
        time_slot = (mda.get('start_time') or
                     mda.get('nominal_time') or
                     mda.get('end_time'))

        # orbit is not given for GEO satellites, use None
        if 'orbit_number' not in mda:
            mda['orbit_number'] = None

        platform = mda["platform_name"]

        self.logger.debug("platform %s time %s", str(platform), str(time_slot))

        if isinstance(mda['sensor'], (list, tuple, set)):
            sensor = mda['sensor'][0]
        else:
            sensor = mda['sensor']

        if sensor not in instruments:
            self.logger.debug("Unknown sensor, skipping data.")
            return None

        # Create satellite scene
        global_data = GF.create_scene(satname=str(platform),
                                      satnumber='',
                                      instrument=str(sensor),
                                      time_slot=time_slot,
                                      orbit=mda['orbit_number'],
                                      variant=mda.get('variant', ''))
        self.logger.info("Creating scene for satellite %s and time %s",
                         str(platform), str(time_slot))

        # Update missing information to global_data.info{}
        # TODO: this should be fixed in mpop.
        global_data.info.update(mda)
        global_data.info['time'] = time_slot

        return global_data


def get_data_fnames(msg):
    """Get input data filenames from the message"""
    # Add list of filenames to info dictionary
    data_fnames = None

    if "uri" in msg.data:
        data_fnames = [urlparse(msg.data["uri"]).path, ]
    elif "dataset" in msg.data:
        data_fnames = \
            [urlparse(itm["uri"]).path for itm in msg.data["dataset"]]
    elif "collection" in msg.data:
        if "uri" in msg.data["collection"][0]:
            data_fnames = \
                [urlparse(itm["uri"]).path for itm in msg.data["collection"]]
        elif "dataset" in msg.data["collection"][0]:
            data_fnames = []
            for dset in msg.data["collection"]:
                for itm in dset["dataset"]:
                    data_fnames.append(urlparse(itm["uri"]).path)

    return data_fnames



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Copyright (c) 2016 Panu Lahtinen

# Author(s):

#   Panu Lahtinen <pnuu+git@iki.fi>

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Setup for trollflow-sat.
"""
from setuptools import setup
import imp

version = imp.load_source('trollflow_sat.version', 'trollflow_sat/version.py')

setup(name="trollflow_sat",
      version=version.__version__,
      description='Pytroll workflow plugins for satellite data',
      author='Panu Lahtinen',
      author_email='panu.lahtinen@fmi.fi',
      classifiers=["Development Status :: 3 - Alpha",
                   "Intended Audience :: Science/Research",
                   "License :: OSI Approved :: GNU General Public License v3 " +
                   "or later (GPLv3+)",
                   "Operating System :: OS Independent",
                   "Programming Language :: Python",
                   "Topic :: Scientific/Engineering"],
      url="https://github.com/pytroll/trollflow-sat",
      packages=['trollflow_sat', ],
      scripts=[],
      data_files=[],
      zip_safe=False,
      install_requires=['pyyaml', 'trollflow', 'posttroll', 'trollsift',
                        'pykdtree', 'mipp', 'mpop'],
      tests_require=['trollflow'],
      test_suite='trollflow_sat.tests.suite',
      )
